from tests import AdaptiveTests
from ipn import TestPaymentIPN, TestPreapprovalIPN, TestIPNVerification
from preapproval_return_url import TestPreapprovalReturnURL
from preapproval_cancel import TestPreapprovalCancel
from preapproval_update import TestPreapprovalUpdate
from payment import PaymentDBTest
from payment_return_url import TestPaymentReturnURL
from payment_response import TestPaymentResponses
from payment_update import TestPaymentUpdate
